#include <math.h>
#include <vector>

using namespace std;

#include "audioeffectx.h"



#define PLUGIN_NAME			"1bitstudio - noisedrums"
#define PLUGIN_VENDOR		"Shiru"
#define PLUGIN_PRODUCT		"v0.14 03.06.19"
#define PLUGIN_UID			'1nsd'

#define NUM_INPUTS			0
#define NUM_OUTPUTS			2
#define NUM_PROGRAMS		8

#define NOISE_MAX_BURSTS	8
#define NOISE_MAX_DURATION	1000
#define NOISE_MAX_PITCH		32000.0f

#define SYNTH_NOTES			12
#define SYNTH_CHANNELS		8

#define OVERSAMPLING		8

#define CHUNK_SIZE			65536

#define MAX_NAME_LEN		32


enum {
	pIdNoteMapping=0,
	pIdBurstNumber,
	pIdDuration,
	pIdPitch,
	pIdPeriod,
	pIdPulseWidth,
	pIdDrumVolume,
	pIdPolyphony,
	pIdOutputGain,
	NUM_PARAMS
};



const char* const programDefaultNames[NUM_PROGRAMS]={
	"Kit 1",
	"User 1",
	"User 2",
	"User 3",
	"User 4",
	"User 5",
	"User 6",
	"User 7"
};



const char* const pNoteNames[12]={
	"C (Kick)",
	"C# (Crash)",
	"D (Snare)",
	"D# (Ride)",
	"E (Clap)",
	"F (Cowbell)",
	"F# (Closed hat)",
	"G (Low tom)",
	"G# (Medium hat)",
	"A (Mid tom)",
	"A# (Open hat)",
	"B (Hi tom)"
};



enum {
	mEventTypeNote=0,
	mEventTypeProgram
};



struct MidiQueueStruct
{
	VstInt32 type;
	VstInt32 delta;
	VstInt32 note;
	VstInt32 velocity;
	VstInt32 program;
};



struct SynthChannelStruct
{
	VstInt32 note;
	VstInt32 burst;

	float duration;
	float accumulator;
	float increment;

	VstInt32 period;
	VstInt32 ptr;

	float volume;

	VstInt32 output;
};



#define TAG(s)		(float)(((s)[0])|(((s)[1])<<7)|(((s)[2])<<14)|(((s)[3]<<21)))

#define DATA		TAG("DATA")
#define PROG		TAG("PROG")
#define NAME		TAG("NAME")
#define BRST		TAG("BRST")

#define NMAP		TAG("NMAP")
#define POLY		TAG("POLY")
#define GAIN		TAG("GAIN")
#define NOTE		TAG("NOTE")
#define VOLU		TAG("VOLU")
#define DURA		TAG("DURA")
#define PTCH		TAG("PTCH")
#define PERD		TAG("PERD")
#define PWDT		TAG("PWDT")

#define DONE		TAG("DONE")



const float ChunkPresetData[8114]={	//if the size is not specified, VS 2017 bloats the dll size from 64K to 512K!

	DATA,

	PROG,0,
	POLY,1.000f,GAIN,1.000f,
	NOTE,0,VOLU,1.000f,
	BRST,0,DURA,0.003f,PTCH,0.558f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.032f,PTCH,0.010f,PERD,0.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,1,VOLU,0.745f,
	BRST,0,DURA,0.055f,PTCH,1.000f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.086f,PTCH,1.000f,PERD,1.000f,PWDT,0.576f,
	BRST,2,DURA,0.172f,PTCH,1.000f,PERD,1.000f,PWDT,0.340f,
	BRST,3,DURA,0.170f,PTCH,1.000f,PERD,1.000f,PWDT,0.109f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,2,VOLU,1.000f,
	BRST,0,DURA,0.020f,PTCH,0.015f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.028f,PTCH,0.533f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.065f,PTCH,1.000f,PERD,1.000f,PWDT,0.117f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,3,VOLU,0.779f,
	BRST,0,DURA,0.097f,PTCH,0.254f,PERD,1.000f,PWDT,0.277f,
	BRST,1,DURA,0.105f,PTCH,0.138f,PERD,1.000f,PWDT,0.119f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,4,VOLU,1.000f,
	BRST,0,DURA,0.012f,PTCH,0.069f,PERD,0.000f,PWDT,1.000f,
	BRST,1,DURA,0.047f,PTCH,0.819f,PERD,1.000f,PWDT,0.814f,
	BRST,2,DURA,0.137f,PTCH,0.831f,PERD,1.000f,PWDT,0.072f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,5,VOLU,1.000f,
	BRST,0,DURA,0.027f,PTCH,0.021f,PERD,0.994f,PWDT,0.137f,
	BRST,1,DURA,0.000f,PTCH,1.000f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,1.000f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,6,VOLU,1.000f,
	BRST,0,DURA,0.036f,PTCH,0.305f,PERD,1.000f,PWDT,0.225f,
	BRST,1,DURA,0.000f,PTCH,1.000f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,7,VOLU,1.000f,
	BRST,0,DURA,0.037f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.099f,PTCH,0.041f,PERD,1.000f,PWDT,0.125f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,8,VOLU,1.000f,
	BRST,0,DURA,0.038f,PTCH,0.303f,PERD,1.000f,PWDT,0.226f,
	BRST,1,DURA,0.044f,PTCH,1.000f,PERD,1.000f,PWDT,0.197f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,9,VOLU,1.000f,
	BRST,0,DURA,0.034f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.066f,PTCH,0.061f,PERD,1.000f,PWDT,0.118f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,10,VOLU,1.000f,
	BRST,0,DURA,0.109f,PTCH,0.306f,PERD,1.000f,PWDT,0.217f,
	BRST,1,DURA,0.071f,PTCH,1.000f,PERD,1.000f,PWDT,0.200f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,11,VOLU,1.000f,
	BRST,0,DURA,0.033f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.073f,PTCH,0.065f,PERD,1.000f,PWDT,0.117f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	PROG,1,
	POLY,1.000f,GAIN,1.000f,
	NOTE,0,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,1,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,2,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,3,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,4,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,5,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,6,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,7,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,8,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,9,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,10,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,11,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	PROG,2,
	POLY,1.000f,GAIN,1.000f,
	NOTE,0,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,1,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,2,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,3,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,4,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,5,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,6,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,7,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,8,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,9,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,10,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,11,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	PROG,3,
	POLY,1.000f,GAIN,1.000f,
	NOTE,0,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,1,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,2,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,3,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,4,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,5,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,6,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,7,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,8,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,9,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,10,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,11,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	PROG,4,
	POLY,1.000f,GAIN,1.000f,
	NOTE,0,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,1,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,2,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,3,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,4,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,5,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,6,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,7,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,8,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,9,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,10,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,11,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	PROG,5,
	POLY,1.000f,GAIN,1.000f,
	NOTE,0,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,1,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,2,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,3,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,4,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,5,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,6,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,7,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,8,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,9,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,10,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,11,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	PROG,6,
	POLY,1.000f,GAIN,1.000f,
	NOTE,0,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,1,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,2,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,3,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,4,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,5,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,6,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,7,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,8,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,9,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,10,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,11,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	PROG,7,
	POLY,1.000f,GAIN,1.000f,
	NOTE,0,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,1,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,2,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,3,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,4,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,5,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,6,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,7,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,8,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,9,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,10,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	NOTE,11,VOLU,1.000f,
	BRST,0,DURA,0.050f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,1,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,2,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,3,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,4,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,5,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,6,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,
	BRST,7,DURA,0.000f,PTCH,0.500f,PERD,1.000f,PWDT,1.000f,

	DONE

};



class NoiseDrums:public AudioEffectX
{
public:

	NoiseDrums(audioMasterCallback audioMaster);

	~NoiseDrums();

	virtual void setParameter(VstInt32 index,float value);
	virtual float getParameter(VstInt32 index);
	virtual void getParameterLabel(VstInt32 index,char *label);
	virtual void getParameterDisplay(VstInt32 index,char *text);
	virtual void getParameterName(VstInt32 index,char *text); 

	virtual VstInt32 getProgram();
	virtual void setProgram(VstInt32 program);
	virtual void getProgramName(char* name);
	virtual void setProgramName(char* name);

	virtual bool getEffectName(char *name) { strcpy(name,PLUGIN_NAME); return true; }
	virtual bool getVendorString(char *text) { strcpy(text,PLUGIN_VENDOR); return true; }
	virtual bool getProductString(char *text) { strcpy(text, PLUGIN_PRODUCT); return true; }
	virtual VstInt32 getVendorVersion() { return 1000; } 

	VstInt32 canDo(char* text);
	VstInt32 getNumMidiInputChannels(void);
	VstInt32 getNumMidiOutputChannels(void);

	virtual VstInt32 getChunk(void** data,bool isPreset=false);
	virtual VstInt32 setChunk(void* data,VstInt32 byteSize,bool isPreset=false);

	VstInt32 processEvents(VstEvents* ev);
	void processReplacing(float**inputs,float **outputs,VstInt32 sampleFrames);

protected:

	VstInt32 Program;

	float pNoteMapping[NUM_PROGRAMS];
	int   pNoteMappingInt;

	float pBurstNumber[NUM_PROGRAMS][SYNTH_NOTES];
	int   pBurstNumberInt;

	float pDuration   [NUM_PROGRAMS][SYNTH_NOTES][NOISE_MAX_BURSTS];
	float pPitch      [NUM_PROGRAMS][SYNTH_NOTES][NOISE_MAX_BURSTS];
	float pPeriod     [NUM_PROGRAMS][SYNTH_NOTES][NOISE_MAX_BURSTS];
	float pPulseWidth [NUM_PROGRAMS][SYNTH_NOTES][NOISE_MAX_BURSTS];

	float pDrumVolume [NUM_PROGRAMS][SYNTH_NOTES];

	float pPolyphony  [NUM_PROGRAMS];
	float pOutputGain [NUM_PROGRAMS];

	char ProgramName  [NUM_PROGRAMS][MAX_NAME_LEN];

	float Chunk[CHUNK_SIZE];

	float FloatToHz(float value,float range);
	int FloatToNoisePeriod(float value);

	VstInt32 SavePresetChunk(float *chunk);
	void LoadPresetChunk(float *chunk);

	VstInt32 SaveStringChunk(char *str,float *dst);
	VstInt32 LoadStringChunk(char *str,int max_length,float *src);

	void MidiAddNewNote(VstInt32 delta,VstInt32 note,VstInt32 velocity);
	void MidiAddProgramChange(VstInt32 delta,VstInt32 program);

	void SynthNewNoiseBurst(VstInt32 chn);

	vector<MidiQueueStruct> MidiQueue;

	SynthChannelStruct SynthChannel[SYNTH_CHANNELS];

	unsigned int Noise[2048];
};
